/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.nau;

import java.io.StringReader;

import at.jku.risc.stout.nau.algo.AntiUnify;
import at.jku.risc.stout.nau.algo.AntiUnifyProblem;
import at.jku.risc.stout.nau.algo.DebugLevel;
import at.jku.risc.stout.nau.data.EquationSystem;
import at.jku.risc.stout.nau.data.FreshnessCtx;
import at.jku.risc.stout.nau.data.InputParser;
import at.jku.risc.stout.nau.data.NodeFactory;
import at.jku.risc.stout.nau.data.atom.NominalTerm;

/**
 * Some test cases
 * 
 * @author Alexander Baumgartner
 */
public class TestAntiUnify {

	public static void main(String[] args) {
		test("f(a,b) =^= f(a,b)");
		test("f(a:i,b:i):o =^= f:i-i-o(a,b)");
		test("f(a:i,a:i):o =^= f:i-i-o(b,b)");
		test("f(a,a) =^= f(b,b)");
		test("f(a,a) =^= f(b,c)");
		test("f(a,b) =^= f(b,c)");
		test("a.a =^= b.c");
		test("a.b =^= b.a");
		test("a.b =^= b.a", "", "c");
		test("f(a,b) =^= f(Y,(a b)Y)", "b#Y");
		test("f(b,a) =^= f(Y,(a b)Y)", "b#Y");
		test("h(a.a, b.b)> =^= h(c.Y, c.Y)>", "a#Y");
		test("h(a.a, b.b)> =^= h(c.Y, c.Y)>", "");
		test("h(f(X1), c.f(X1)) =^= h(g(X2), c.g(X2))", "c#X1, c#X2");
		test("h(f(X1), a.f((a c)X1)) =^= h(g(X2), b.g((b c)X2))",
				"a#X1,b#X1,c#X1,a#X2,b#X2,c#X2");
		test("h(f(X1), a.f(X2)) =^= h(g(X1), a.g(X2))", "a#X1");
		test("a.f(a,b) =^= b.f(b,a)", "");
		test("a.f(a,b) =^= b.f(b,a)", "", "c");
		test("a.b.c =^= c.a.b");
		test("a.b.b =^= b.b.a");
		test("f(a.b, X) =^= f(b.a, Y)", "c#X");
		test("h(f(X1), a.a.f((a c)X1)) =^= h(g(X2), c.b.g((b c)X2))",
				"{a#X1,b#X1,c#X1}");
	}

	public static void test(String eqSysIn) {
		test(eqSysIn, "");
	}

	public static void test(String eqSysIn, String nablaIn) {
		test(eqSysIn, nablaIn, "");
	}

	public static void test(String eqSysIn, String nablaInS, String extraAtoms) {
		try {
			final NodeFactory factory = new NodeFactory();
			EquationSystem<AntiUnifyProblem> eqSys = new EquationSystem<AntiUnifyProblem>() {
				@Override
				public AntiUnifyProblem newEquation(NominalTerm left,
						NominalTerm right) {
					return new AntiUnifyProblem(left, right, factory);
				}
			};
			InputParser parser = new InputParser(factory);
			parser.parseEquationSystem(new StringReader(eqSysIn), eqSys);
			FreshnessCtx nablaIn = parser
					.parseNabla(new StringReader(nablaInS));
			parser.parseAtomSet(new StringReader(extraAtoms));
			AntiUnify au = new AntiUnify(eqSys, nablaIn, DebugLevel.PROGRESS,
					factory);
			System.out.print("Nabla in: ");
			nablaIn.printString(System.out);
			System.out.print(" Atoms: ");
			System.out.println(factory.getAllByType(factory.classAtom));
			System.out.println();
			au.antiUnify(true, System.out);
			System.out.println();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

}
