package at.jku.risc.stout.nau;

import java.io.IOException;
import java.io.Reader;
import java.io.StringReader;
import java.util.Collection;

import at.jku.risc.stout.nau.algo.AntiUnify;
import at.jku.risc.stout.nau.algo.AntiUnifyProblem;
import at.jku.risc.stout.nau.algo.AntiUnifySystem;
import at.jku.risc.stout.nau.algo.DebugLevel;
import at.jku.risc.stout.nau.algo.Equivariance;
import at.jku.risc.stout.nau.algo.EquivarianceProblem;
import at.jku.risc.stout.nau.algo.JustificationException;
import at.jku.risc.stout.nau.data.EquationSystem;
import at.jku.risc.stout.nau.data.FreshnessCtx;
import at.jku.risc.stout.nau.data.InputParser;
import at.jku.risc.stout.nau.data.NodeFactory;
import at.jku.risc.stout.nau.data.atom.Atom;
import at.jku.risc.stout.nau.data.atom.NominalTerm;
import at.jku.risc.stout.nau.data.atom.Permutation;
import at.jku.risc.stout.nau.data.atom.Variable;
import at.jku.risc.stout.nau.util.ControlledException;

public class TestPaperExample {

	public static void main(String[] args) throws IOException,
			JustificationException, ControlledException {
		auTest();
		equivTest();
	}

	public static void auTest() throws IOException, ControlledException {
		Reader in1 = new StringReader("a.f(a,a,b)"); // Use FileReader instead
		Reader in2 = new StringReader("a.f(a,b,b)"); // Use FileReader instead
		Reader inN = new StringReader("a#X"); // Use FileReader instead
		Reader inA = new StringReader("c,d"); // Use FileReader instead

		final NodeFactory factory = new NodeFactory();
		EquationSystem<AntiUnifyProblem> eqSys = new EquationSystem<AntiUnifyProblem>() {
			public AntiUnifyProblem newEquation(NominalTerm t, NominalTerm s) {
				return new AntiUnifyProblem(t, s, factory);
			}
		};
		FreshnessCtx nablaIn = new InputParser(factory).parseEquationAndCtx(
				in1, in2, inA, inN, eqSys);

		new AntiUnify(eqSys, nablaIn, DebugLevel.SILENT, factory) {
			public void callback(AntiUnifySystem res, Variable var) {
				System.out.println(res.getNablaGen());
				System.out.println(res.getSigma().get(var));
			};
		}.antiUnify(false, null);
	}

	public static void equivTest() throws IOException, ControlledException {
		Reader in1 = new StringReader("a.f(a,c,b,X)"); // Use FileReader instead
		Reader in2 = new StringReader("a.f(a,b,c,X)"); // Use FileReader instead
		Reader inN = new StringReader("b#X,c#X"); // Use FileReader instead

		final NodeFactory factory = new NodeFactory();
		InputParser parser = new InputParser(factory);

		EquationSystem<EquivarianceProblem> eqSys = new EquationSystem<EquivarianceProblem>() {
			public EquivarianceProblem newEquation(NominalTerm t, NominalTerm s) {
				return new EquivarianceProblem(t, s);
			}
		};
		parser.parseEquation(in1, in2, eqSys);

		FreshnessCtx nablaIn = parser.parseNabla(inN);
		Collection<? extends Atom> atomSet = factory
				.getAllByType(factory.classAtom);

		Permutation pi = new Equivariance(eqSys, atomSet, nablaIn).compute(
				factory, false, DebugLevel.SILENT, null);
		System.out.println(pi);
	}
}
