/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.nau;

import java.io.IOException;
import java.io.StringReader;

import at.jku.risc.stout.nau.algo.AntiUnifyProblem;
import at.jku.risc.stout.nau.data.EquationSystem;
import at.jku.risc.stout.nau.data.FreshnessCtx;
import at.jku.risc.stout.nau.data.InputParser;
import at.jku.risc.stout.nau.data.NodeFactory;
import at.jku.risc.stout.nau.data.atom.Atom;
import at.jku.risc.stout.nau.data.atom.NominalTerm;
import at.jku.risc.stout.nau.util.ControlledException;

/**
 * Some test cases
 * 
 * @author Alexander Baumgartner
 */
public class TestParser {

	public static void main(String[] args) {
		try {
			test();
		} catch (Exception e) {
			e.printStackTrace();
		}
	}

	public static void test() throws Exception {
		parseNabla("");
		parseNabla("a#Xalhg");
		parseNabla("a#X,b#Y");
		parseNabla(" \n\t   a#\nX, b# Y ");
		parseNabla("{c#X,b#Y}");
		parseTerm("a.f(a,b,(a b)X)");
		parseTerm("a.f[a:i,b:i,(a b)X:i]");
		parseTerm("a.f(a:i,b:i,(a b)X:i):o");
		parseTerm(" a.f:i-i-i-o(a,b,(a b)X)    ");
		parseTerm("a:i.f:i-i-i-o(a:i,b:i,(a:i b:i)X:i):o");
		parseTerm("a.f:i-i-i-o(a,b,(a b)X):o");
		parseTerm("  (a b)(c d)(a d)(af e)Y");
		parseTerm("(a:i b:i)(c d)(a d)(af e)Y");
		parseTerm(" (a b)(c d)(a d)(af e)Y  ");
		parseTerm("(a b)(c d)(a d)(af e)Y");
		parseEquationSystem("a=b");
		parseEquationSystem("a=^=b");
		parseEquationSystem("a =^= \n b");
		parseEquationSystem("a=b;c =^= \n b");
		parseEquationSystem("a.f(a,b,(a b)X)=^= a.f[a:i,b:i,(a b)X:i],"
				+ "a.f(a:i,b:i,(a b)X:i):o=a.f:i-i-i-o(a,b,(a b)X),"
				+ "a:i.f:i-i-i-o(a:i,b:i,(a:i b:i)X:i):o=a.f:i-i-i-o(a,b,(a b)X):o,"
				+ "  (a b)(c d)(a d)(af e)Y=(a:i b:i)(c d)(a d)(af e)Y,"
				+ "(a b)(c d)(a d)(af e)Y= (a b)(c d)(a d)(af e)Y");
	}

	private static void parseTerm(String nomTerm) throws Exception {
		NodeFactory factory = new NodeFactory();
		InputParser nParse = new InputParser(factory);
		NominalTerm np = nParse.parseTerm(new StringReader(nomTerm), true);
		np.printString(System.out);
		System.out.print(" ==> SWAP (a c) ==> ");
		Atom a1 = factory.newAtom("a");
		Atom a2 = factory.newAtom("c");
		np.swap(a1, a2);
		np.printString(System.out);
		System.out.println();
	}

	private static void parseNabla(String input) throws Exception {
		NodeFactory factory = new NodeFactory();
		InputParser nParse = new InputParser(factory);
		FreshnessCtx nabla = nParse.parseNabla(new StringReader(input));
		nabla.printString(System.out);
		System.out.print(" ==> SWAP (a c) ==> ");
		Atom a1 = factory.newAtom("a");
		Atom a2 = factory.newAtom("c");
		nabla.swap(a1, a2).printString(System.out);
		System.out.println();
	}

	private static void parseEquationSystem(String nomEqSysIn)
			throws IOException, ControlledException {
		final NodeFactory factory = new NodeFactory();
		InputParser nParse = new InputParser(factory);
		EquationSystem<AntiUnifyProblem> nomEqSysOut = new EquationSystem<AntiUnifyProblem>() {
			@Override
			public AntiUnifyProblem newEquation(NominalTerm left,
					NominalTerm right) {
				return new AntiUnifyProblem(left, right, factory);
			}
		};
		nParse.parseEquationSystem(new StringReader(nomEqSysIn), nomEqSysOut);
		nomEqSysOut.printString(System.out);
		System.out.println();
	}
}
