/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.nau.algo;

import java.io.IOException;
import java.io.Writer;

import at.jku.risc.stout.nau.data.Equation;
import at.jku.risc.stout.nau.data.NominalPair;
import at.jku.risc.stout.nau.data.atom.NominalTerm;

/**
 * This class represents an equivariance problem (an equation of the form t = s)
 * of two {@linkplain NominalPair}s. A permutation is computed by the rule based
 * system {@linkplain EquivarianceSystem}. It is a sub algorithm of the rule
 * based anti-unification algorithm {@linkplain AntiUnifySystem}.
 * 
 * @author Alexander Baumgartner
 */
public class EquivarianceProblem extends Equation<NominalTerm> {
	public static String eqSeparator = " = ";

	public EquivarianceProblem(NominalTerm left, NominalTerm right) {
		super(left, right);
	}

	@Override
	public void printString(Writer toPrint) throws IOException {
		printString(toPrint, getLeft(), getRight());
	}

	public static void printString(Writer toPrint, NominalTerm left,
			NominalTerm right) throws IOException {
		left.printString(toPrint);
		toPrint.write(eqSeparator);
		right.printString(toPrint);
	}

	@Override
	public Equation<NominalTerm> deepCopy() {
		return new EquivarianceProblem(getLeft().deepCopy(), getRight()
				.deepCopy());
	}
}