/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.nau.data;

import at.jku.risc.stout.nau.data.atom.NominalTerm;
import at.jku.risc.stout.nau.util.DeepCopy;
import at.jku.risc.stout.nau.util.Printable;

/**
 * An equation consists of two {@linkplain NominalTerm}s.
 * 
 * @author Alexander Baumgartner
 */
public abstract class Equation<T extends Printable> extends Printable implements
		DeepCopy<Equation<T>> {
	private T left, right;

	/**
	 * An equation consists of two terms, which are called the left and the
	 * right part of the equation, respectively.
	 */
	public Equation(T left, T right) {
		this.left = left;
		this.right = right;
	}

	/**
	 * Returns the left part of the equation.
	 */
	public T getLeft() {
		return left;
	}

	/**
	 * Changes the left part of the equation.
	 */
	public void setLeft(T left) {
		this.left = left;
	}

	/**
	 * Returns the right part of the equation.
	 */
	public T getRight() {
		return right;
	}

	/**
	 * Changes the right part of the equation.
	 */
	public void setRight(T right) {
		this.right = right;
	}
}
