/*
 * Copyright 2013 Alexander Baumgartner
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU Lesser General Public License as published
 * by the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU Lesser General Public License for more details.
 *
 * You should have received a copy of the GNU Lesser General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */

package at.jku.risc.stout.nau.data.atom;

import at.jku.risc.stout.nau.data.NodeFactory;

/**
 * A variable has a unique name and may be of sort data or sort atom.
 * 
 * @author Alexander Baumgartner
 */
public class Variable implements HasSort<Sort> {
	private String name;
	private Sort sort;

	/**
	 * Instantiates a variable of the specified name. Use the
	 * {@linkplain NodeFactory} to create new {@linkplain Variable}s! The use of
	 * {@linkplain NodeFactory} guarantees the uniqueness of equal
	 * {@linkplain Variable}s. This uniqueness enables very fast equality
	 * testing, hash-code generation, comparison of variables,... and it keeps
	 * the memory overhead as small as possible.
	 */
	public Variable(String name) {
		this.name = name;
	}

	public String getName() {
		return name;
	}

	public void setName(String name) {
		this.name = name;
	}

	@Override
	public Sort getSort() {
		return sort;
	}

	@Override
	public void setSort(Sort sort) {
		this.sort = sort;
	}

	@Override
	public String toString() {
		return sort == null ? name : name + ":" + sort;
	}
}
